package jerozgen.languagereload.mixin;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableList.Builder;
import com.google.common.collect.Maps;
import jerozgen.languagereload.access.ITranslationStorage;
import jerozgen.languagereload.config.Config;
import net.minecraft.class_1078;
import net.minecraft.class_2477;
import net.minecraft.class_2588;
import net.minecraft.class_2590;
import net.minecraft.class_5348;
import net.minecraft.class_7417;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Consumer;

@Mixin(class_2588.class)
abstract class TranslatableTextContentMixin implements class_7417 {
    @Unique private @Nullable String previousTargetLanguage;
    @Unique private final Map<String, List<class_5348>> separateTranslationsCache = Maps.newHashMap();
    @Unique private @Nullable List<class_5348> savedTranslations;

    @Shadow @Final private String key;
    @Shadow private @Nullable class_2477 languageCache;
    @Shadow private List<class_5348> translations;

    @Inject(method = "updateTranslations", at = @At("RETURN"))
    void onUpdateTranslations(CallbackInfo ci) {
        if (Config.getInstance() == null) return;
        if (!Config.getInstance().multilingualItemSearch) return;
        if (!(languageCache instanceof class_1078)) return;

        var targetLanguage = ((ITranslationStorage) languageCache).languagereload_getTargetLanguage();
        if (Objects.equals(previousTargetLanguage, targetLanguage)) return;

        if (targetLanguage == null) {
            previousTargetLanguage = null;
            translations = savedTranslations;
            savedTranslations = null;
            return;
        }

        if (previousTargetLanguage == null) {
            savedTranslations = translations;
        }
        previousTargetLanguage = targetLanguage;
        translations = separateTranslationsCache.computeIfAbsent(targetLanguage, k -> {
            var string = languageCache.method_48307(key);
            try {
                var builder = new ImmutableList.Builder<class_5348>();
                this.forEachPart(string, builder::add);
                return builder.build();
            } catch (class_2590 e) {
                return ImmutableList.of(class_5348.method_29430(string));
            }
        });
    }

    @Inject(method = "updateTranslations", at = @At(value = "INVOKE",
            target = "Lnet/minecraft/util/Language;get(Ljava/lang/String;)Ljava/lang/String;"))
    void onUpdateTranslations$clearCache(CallbackInfo ci) {
        previousTargetLanguage = null;
        separateTranslationsCache.clear();
        savedTranslations = null;
    }

    @Shadow protected abstract void forEachPart(String translation, Consumer<class_5348> partsConsumer);
}
